Package: auth0
Type: Package
Title: Authentication in Shiny with Auth0
Version: 0.3.0
Authors@R: c(
    person("Julio", "Trecenti", email = "julio.trecenti@gmail.com", role = "cre"),
    person("Daniel", "Falbel", email = "dfalbel@gmail.com", role = "aut"),
    person("José", "Jesus", email = "jjesusfilho@gmail.com", role = "ctb"),
    person("Dean", "Attali", email = "daattali@gmail.com", role = "ctb"),
    person("William", "Amorim", email = "TheWilliam89@gmail.com", role = "ctb"),
    person("C", "Lente", email = "c@lente.dev", role = "ctb"))
Description: Uses Auth0 API (see <https://auth0.com> for more 
    information) to use a simple authentication system. It provides 
    tools to log in and out a shiny application using social networks or a list
    of e-mails.
BugReports: https://github.com/curso-r/auth0/issues
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: httr, shiny, yaml, utils, shinyjs
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
URL: https://curso-r.github.io/auth0/
NeedsCompilation: no
Packaged: 2026-04-13 16:37:49 UTC; jtrec
Author: Julio Trecenti [cre],
  Daniel Falbel [aut],
  José Jesus [ctb],
  Dean Attali [ctb],
  William Amorim [ctb],
  C Lente [ctb]
Maintainer: Julio Trecenti <julio.trecenti@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-13 19:20:09 UTC
Built: R 4.5.2; ; 2026-04-15 05:07:46 UTC; unix

# Generated by roxygen2: do not edit by hand

export(auth0_config)
export(auth0_find_config_file)
export(auth0_info)
export(auth0_logout_url)
export(auth0_server)
export(auth0_ui)
export(logout)
export(logoutButton)
export(logout_url)
export(shinyAppAuth0)
export(shinyAuth0App)
export(use_auth0)

library(shiny)
library(auth0)

# # run this before running the app:
# options(shiny.port = 8080)
# shiny::enableBookmarking(store = "server")

server <- function(input, output, session) {
  output$out <- renderText({
    if (input$caps)
      toupper(input$txt)
    else
      input$txt
  })
}

auth0_server(server)

library(shiny)
library(auth0)

ui <- bootstrapPage(
  fluidRow(plotOutput("plot"))
)

server <- function(input, output, session) {
  output$plot <- renderPlot({
    plot(1:10)
  })
}

shinyAppAuth0(ui, server)

library(shiny)
library(auth0)

# # run this before running the app:
# options(shiny.port = 8080)
# shiny::enableBookmarking(store = "server")

# this example is the same as bookmark, but using bookmarking
ui <- function(request) {
  fluidPage(
    textInput("txt", "Enter text"),
    checkboxInput("caps", "Capitalize"),
    verbatimTextOutput("out"),
    bookmarkButton(),
    logoutButton()
  )
}

auth0_ui(ui)

library(shiny)
library(auth0)

# simple UI with user info
ui <- fluidPage(
  verbatimTextOutput("user_info")
)

# server with one observer that logouts
server <- function(input, output, session) {

  # print user info
  output$user_info <- renderPrint({
    session$userData$auth0_info
  })

}

shinyAppAuth0(ui, server)

library(shiny)
library(auth0)

ui <- function(request) {
  fluidPage(
    textInput("txt", "Enter text"),
    checkboxInput("caps", "Capitalize"),
    verbatimTextOutput("out"),
    bookmarkButton()
  )
}
server <- function(input, output, session) {
  output$out <- renderText({
    if (input$caps)
      toupper(input$txt)
    else
      input$txt
  })
}

enableBookmarking(store = "server")
shinyAppAuth0(ui, server)

#' Modifies ui/server objects to authenticate using Auth0.
#'
#' These functions can be used in a ui.R/server.R framework, modifying the
#'   shiny objects to authenticate using Auth0 service with no pain.
#'
#' @param ui `shiny.tag.list` object to generate the user interface.
#'
#' @name ui-server
#'
#' @seealso [auth0_info].
#'
#' @examples
#' \donttest{
#' # first, create the yml file using use_auth0() function
#'
#' if (interactive()) {
#'
#'   # ui.R file
#'   library(shiny)
#'   library(auth0)
#'   auth0_ui(fluidPage(logoutButton()))
#'
#'   # server.R file
#'   library(auth0)
#'   auth0_server(function(input, output, session) {})
#'
#'   # console
#'   options(shiny.port = 8080)
#'   shiny::runApp()
#' }
#'
#' }
#' @export
auth0_ui <- function(ui, info) {
  disable <- getOption("auth0_disable")
  if (!is.null(disable) && disable) {
    ui
  } else {
    if (missing(info)) info <- auth0_info()
    function(req) {
      verify <- has_auth_code(shiny::parseQueryString(req$QUERY_STRING), info$state)
      if (!verify) {
        if (grepl("error=unauthorized", req$QUERY_STRING)) {
          redirect <- sprintf("location.replace(\"%s\");", logout_url())
          shiny::tags$script(shiny::HTML(redirect))
        } else {

          params <- shiny::parseQueryString(req$QUERY_STRING)
          params$code <- NULL
          params$state <- NULL

          query_params <- paste(
            mapply(paste, names(params), params, MoreArgs = list(sep = "=")),
            collapse = "&")
          query <- if (query_params != "") paste0("/?", query_params) else "/"
          if (!is.null(info$remote_url) && info$remote_url != "" && !getOption("auth0_local")) {
            redirect_uri <- paste0(info$remote_url, query)
          } else {
            if (grepl("127.0.0.1", req$HTTP_HOST)) {
              redirect_uri <- paste0("http://", gsub("127.0.0.1", "localhost", req$HTTP_HOST), query)
            } else {
              redirect_uri <- paste0("http://", req$HTTP_HOST, query)
            }
          }
          redirect_uri <<- redirect_uri
          query_extra <- if(!is.null(info$audience) || !is.null(info$extra_params)) {
            c(info$extra_params, list(audience = info$audience))
          } else {
            NULL
          }
          url <- httr::oauth2.0_authorize_url(
            info$api, info$app(redirect_uri), scope = info$scope, state = info$state,
            query_extra=query_extra
          )
          redirect <- sprintf("location.replace(\"%s\");", url)
          shiny::tags$script(shiny::HTML(redirect))
        }
      } else {
        if (is.function(ui)) {
          ui(req)
        } else {
          ui
        }
      }
    }
  }
}

#' @rdname ui-server
#'
#' @param server the shiny server function.
#' @param info object returned from [auth0_info]. If not informed,
#'   will try to find the `_auth0.yml` and create it automatically.
#' @param remove_callback_params whether to remove the `code` and `state` query.
#'
#' @export
auth0_server <- function(server, info, remove_callback_params = TRUE) {
  disable <- getOption("auth0_disable")
  if (!is.null(disable) && disable) {
    server
  } else {
    if (missing(info)) info <- auth0_info()
    function(input, output, session) {
      shiny::isolate(auth0_server_verify(session, info$app, info$api, info$state))
      if (remove_callback_params) {
        auth0_remove_callback_params(session)
      }
      shiny::observeEvent(input[["._auth0logout_"]], logout())
      server(input, output, session)
    }
  }
}

#' Create a Shiny app object with Auth0 Authentication
#'
#' This function modifies ui and server objects to run using Auth0
#' authentication.
#'
#' @param ui an ordinary UI object to create shiny apps.
#' @param server an ordinary server object to create shiny apps.
#' @param config_file path to YAML configuration file.
#' @param remove_callback_params whether to remove the `code` and `state` query 
#' parameters from the URL after successful authentication. Defaults to `TRUE`.
#' @param ... Other arguments passed on to [shiny::shinyApp()].
#'
#' @details
#' You can also use a diferent configuration file by setting the
#' `auth0_config_file` option with:
#' `options(auth0_config_file = "path/to/file.yaml")`.
#'
#' @section Disable auth0 while developing apps:
#'
#' Sometimes, using auth0 to develop and test apps can be frustrating,
#'   because every time the app is started, auth0 requires the user to log-in.
#'   To avoid this problem, one can run `options(auth0_disable = TRUE)` to
#'   disable auth0 temporarily.
#'
#' @export
shinyAppAuth0 <- function(ui, server, config_file = NULL, remove_callback_params = TRUE, ...) {

  disable <- getOption("auth0_disable")
  if (!is.null(disable) && disable) {
    shiny::shinyApp(ui, server, ...)
  } else {
    if (is.null(config_file)) {
      config_file <- auth0_find_config_file()
    }
    info <- auth0_info(config_file)
    shiny::shinyApp(
      auth0_ui(ui, info), 
      auth0_server(server, info, remove_callback_params), 
      ...
    )
  }
}

#' Create a Shiny app object with Auth0 Authentication
#'
#' @description
#'
#' As of auth0 0.1.2, `shinAuth0App()` has
#' been renamed to [shinyAppAuth0()] for consistency.
#'
#' @inheritParams shinyAppAuth0
#'
#' @export
shinyAuth0App <- function(ui, server, config_file = NULL) {
  warning("`shinyAuth0App()` is soft-deprecated as of auth0 0.1.2.",
          "Please use `shinyAppAuth0()` instead.")
  shinyAppAuth0(ui, server, config_file)
}

#' Generate logout URL
#'
#' `auth0_logout_url()` is defunct as of auth0 0.1.2 in order to simplifly the
#'   user experience with the [logoutButton()] function.
#'
#' @param config_file Path to YAML configuration file.
#' @param redirect_js include javascript code to redirect page? Defaults to `TRUE`.
#'
#' @examples
#' \donttest{
#'
#' # simple UI with action button
#' # AFTER auth0 0.1.2
#'
#' if (interactive()) {
#'   library(shiny)
#'   library(auth0)
#'
#'   ui <- fluidPage(logoutButton())
#'   server <- function(input, output, session) {}
#'   config_file <- system.file("simple/_auth0.yml", package = "auth0")
#'   shinyAppAuth0(ui, server, config_file)
#'
#'   # simple UI with action button
#'   # BEFORE auth0 0.1.2
#'
#'   library(shiny)
#'   library(auth0)
#'   library(shinyjs)
#'
#'   # note that you must include shinyjs::useShinyjs() for this to work
#'   ui <- fluidPage(shinyjs::useShinyjs(), actionButton("logout_auth0", "Logout"))
#'
#'   # server with one observer that logouts
#'   server <- function(input, output, session) {
#'     observeEvent(input$logout_auth0, {
#'       # javascript code redirecting to correct url
#'       js <- auth0_logout_url()
#'       shinyjs::runjs(js)
#'     })
#'   }
#'   shinyAuth0App(ui, server, config_file)
#' }
#'
#' }
#'
#'
#' @export
auth0_logout_url <- function(config_file = NULL, redirect_js = TRUE) {

  stop("`auth0_logout_url()` is deprecated. ",
       "See `?logoutButton()` to add a logout button in auth0 apps.")

}

.onLoad <- function(libname, pkgname) {
  op <- options()
  op.auth0 <- list(
    auth0_config_file = NULL,
    auth0_disable = NULL,
    auth0_local = interactive()
  )
  toset <- !(names(op.auth0) %in% names(op))
  if (any(toset)) options(op.auth0[toset])
  invisible()
}

#' Find the configuration file.
#'
#' Tries to find the path to the `_auth0.yml` file. First, it tries to get
#'   this info from `options(auth0_config_file = )`. If this option is `NULL`
#'   (the default) it tries to find the `_auth0.yml` within the working
#'   directory. If the file does not exist, it raises an error.
#'
#' @return Character vector of length one contaning the path of the
#'   `_auth0.yml` file.
#'
#' @seealso [`use_auth0`].
#'
#' @export
auth0_find_config_file <- function() {
  config_file <- getOption("auth0_config_file")

  if (is.null(config_file) || !file.exists(config_file)) {
    config_file <- "./_auth0.yml"
  }

  if (!file.exists(config_file)) {
    stop(
      "Didn't find any YML configuration file. ",
      "There are two possible explanations:\n",
      "1. You didn't create an _auth0.yml file. Solution: Run `use_auth0()`\n",
      "2. You created an _auth0.yml file, but it was not found.\n",
      "You have two options:\n",
      "  Solution 2a): set the path for the _auth0.yml ",
      "file running `options(auth0_config_file = \"/path/to/_auth0.yml\")`. ",
      "Always use absolute path, because shiny::runApp() modifies ",
      "the working directory.\n",
      "  Solution 2b): If your app.R file is in the same directory as the ",
      "_auth0.yml file, set the working directory to the folder ",
      "where _auth0.yml file is located."
    )
  }

  config_file
}

auth0_remove_callback_params <- function(session) {
  shiny::observeEvent(session[["clientData"]]$url_search,
    {
      params <- shiny::parseQueryString(session[["clientData"]]$url_search)

      # Remove only auth callback params and keep any other query params untouched.
      if (!is.null(params$code) || !is.null(params$state)) {
        params$code <- NULL
        params$state <- NULL

        path <- session[["clientData"]]$url_pathname
        hash <- session[["clientData"]]$url_hash

        query <- ""
        if (length(params) > 0) {
          encoded <- mapply(
            function(nm, val) {
              paste0(
                utils::URLencode(nm, reserved = TRUE),
                "=",
                utils::URLencode(as.character(val), reserved = TRUE)
              )
            },
            names(params),
            params,
            SIMPLIFY = TRUE,
            USE.NAMES = FALSE
          )
          query <- paste0("?", paste(encoded, collapse = "&"))
        }

        shiny::updateQueryString(paste0(path, query, hash), mode = "replace", session = session)
      }
    },
    once = TRUE,
    ignoreInit = FALSE
  )
}

# Get rid of NOTE
globalVariables(c("redirect_uri"))

auth0_app <- function(app_url, app_name, key, secret) {
  function(app_url) {
    httr::oauth_app(appname = app_name, key = key, secret = secret, redirect_uri = app_url)
  }
}

auth0_api <- function(auth0_url, request, access) {
  httr::oauth_endpoint(base_url = auth0_url, request = request,
                       authorize = "authorize", access = access)
}

has_auth_code <- function(params, state) {
  is.null(params$error) && !is.null(params$code) && params$state == state
}

auth0_server_verify <- function(session, app, api, state) {

  u_search <- session[["clientData"]]$url_search
  params <- shiny::parseQueryString(u_search)

  if (has_auth_code(params, state)) {
    cred <- httr::oauth2.0_access_token(api, app(redirect_uri), params$code)
    token <- httr::oauth2.0_token(
      app = app(redirect_uri), endpoint = api, cache = FALSE, credentials = cred,
      user_params = list(grant_type = "authorization_code"))

    userinfo_url <- sub("authorize", "userinfo", api$authorize)
    resp <- httr::RETRY(
      verb = "GET"
      , url = userinfo_url
      , httr::config(token = token)
      , times = 5
    )

    assign("auth0_credentials", token$credentials, envir = session$userData)
    assign("auth0_info", httr::content(resp, "parsed"), envir = session$userData)
  }

}

auth0_state <- function(server) {
  paste(sample(c(letters, LETTERS, 0:9), 10, replace = TRUE), collapse = "")
}

#' Information used to connect to Auth0.
#'
#' Creates a list containing all the important information to connect to Auth0
#'   service's API.
#'
#' @param config path to the `_auth0.yml` file or the object returned by
#'   [auth0_config]. If not informed, will try to find the file using
#'   [auth0_find_config_file].
#'
#' @seealso [use_auth0] to create an `_auth0.yml` template.
#'
#' @return A list contaning scope, state, keys, OAuth2.0 app, endpoints,
#'   audience and
#'   remote URL. For compatibility reasons, `remote_url` can be either a parameter
#'   in the root yaml or inside a `shiny_config` parameter.
#'
#' @export
auth0_info <- function(config) {
  if (missing(config)) config <- auth0_config()
  if (!is.list(config) && is.character(config)) config <- auth0_config(config)
  scope <- config$auth0_config$scope
  state <- auth0_state()
  conf <- config$auth0_config
  app <- auth0_app(app_name = config$name, key = conf$credentials$key, secret = conf$credentials$secret)
  api <- auth0_api(conf$api_url, conf$request, conf$access)
  audience <- conf$audience
  rurl <- config$remote_url
  extra_params <- config$request_extra_params
  # backward compatibility
  if (is.null(rurl)) rurl <- config$shiny_config$remote_url
  list(scope = scope, state = state, app = app, api = api, audience = audience,
       remote_url = rurl, extra_params = extra_params)
}

#' Parse `_auth0.yml` file.
#'
#' Validates and creates a list of useful information from
#'   the `_auth0.yml` file.
#'
#' @param config_file path to the `_auth0.yml` file. If not informed,
#'   will try to find the file using [auth0_find_config_file].
#'
#' @return List containing all the information from the `_auth0.yml` file.
#'
#' @export
auth0_config <- function(config_file) {
  if (missing(config_file)) config_file <- auth0_find_config_file()
  config <- yaml::read_yaml(config_file, eval.expr = TRUE)

  # standardise and validate auth0_config
  if (is.null(config$auth0_config)) {
    stop("Missing 'auth0_config' tag in YAML file.")
  }
  config_names <- names(unlist(config$auth0_config))
  required_names <- c("api_url", "credentials.key", "credentials.secret")
  missing_args <- setdiff(required_names, config_names)
  s <- strrep("s", max(length(missing_args) - 1L, 0))
  if (length(missing_args) > 0) {
    msg <- sprintf("Missing '%s' tag%s in YAML file", paste(missing_args, collapse = "','"), s)
    stop(msg)
  }
  # scope
  scp <- config$auth0_config$scope
  if (is.null(scp)) scp <- "openid profile"
  defaults <- list(scope = scp, request = "oauth/token", access = "oauth/token")

  for (nm in names(defaults)) {
    if (!nm %in% config_names) {
      config$auth0_config[[nm]] <- defaults[[nm]]
    }
  }
  config
}

#' Auth0 configuration file
#'
#' Create an YAML containing information to connect with Auth0.
#'
#' @param path Directory name. Should be the root of the shiny app
#' you want to add this functionality
#' @param file File name. Defaults to `_auth0.yml`.
#' @param overwrite Will only overwrite existing path if `TRUE`.
#'
#' @details The YAML configuration file has required parameters and extra
#' parameters.
#'
#' The required parameters are:
#' - `auth0_config` is a list contaning at least:
#'   - `api_url`: Your account at Auth0 (e.g. https://jonhdoe.auth0.com).
#'   It is the "Domain" in Auth0 application settings.
#'   - `credentials`: Your credentials to access Auth0 API, including
#'     - `key`: the Client ID in Auth0 application settings.
#'     - `secret`: the Client Secret in Auth0 application settings.
#'
#' The extra parameters are:
#' - `remote_url`: If you are using Shiny-Server or ShinyApps IO service.
#' - `scope`: The information that Auth0 app will access.
#' Defaults to "openid profile".
#' - `request`: Endpoit to request a token. Defaults to "oauth/token"
#' - `access`: Endpoit to access. Defaults to "oauth/token"
#'
#' @export
use_auth0 <- function(path = ".", file = "_auth0.yml", overwrite = FALSE) {
  f <- paste0(normalizePath(path), "/", file)
  if (file.exists(f) && !overwrite) {
    stop("File exists and overwrite is FALSE.")
  }
  ks <- list(key = 'Sys.getenv("AUTH0_KEY")', secret = 'Sys.getenv("AUTH0_SECRET")')
  api_url <- "paste0('https://', Sys.getenv('AUTH0_USER'), '.auth0.com')"
  attr(ks[[1]], "tag") <- "!expr"
  attr(ks[[2]], "tag") <- "!expr"
  attr(api_url, "tag") <- "!expr"
  yaml_list <- list(
    name = "myApp",
    remote_url = "",
    auth0_config = list(api_url = api_url, credentials = ks))
  yaml::write_yaml(yaml_list, f)
}

#' Create a button to log out
#'
#' A \code{logoutButton} is an \code{\link[shiny]{actionButton}} that is meant
#' to be used to log out of an auth0 Shiny app.
#'
#' @param label The label on the button.
#' @param ... Named attributes to apply to the button.
#' @param id An ID for the button. If you only have one logout button in
#' your app, you do not need to explicitly provide an ID. If you have more than
#' one logout button, you need to provide a unique ID to each button. When you
#' create a button with a non-default ID, you must create an observer that
#' listens to a click on this button and logs out of the app with a call to
#' \code{\link[auth0]{logout}}.
#' @seealso \code{\link[auth0]{logout}},
#' \code{\link[auth0]{logout_url}}
#' @examples
#' if (interactive()) {
#'   ui <- fluidPage(
#'     logoutButton(),
#'     logoutButton(label = "Another logout button", id = "logout2")
#'   )
#'   server <- function(input, output, session) {
#'     observeEvent(input$logout2, {
#'       logout()
#'     })
#'   }
#'   shinyAuth0App(ui, server)
#' }
#'
#' @export
logoutButton <- function(label = "Log out", ..., id = "._auth0logout_") {
  shiny::actionButton(id, label, ...)
}

#' Generate a logout URL
#'
#' Generate a URL that will log the user out of the app if visited.
#'
#' @return URL string to log out.
#'
#' @seealso \code{\link[auth0]{logoutButton}},
#' \code{\link[auth0]{logout}}
#'
#' @details You can also use a diferent configuration file by setting the
#' `auth0_config_file` option with:
#' `options(auth0_config_file = "path/to/file.yaml")`.
#'
#' @export
logout_url <- function() {

  config <- auth0_config()

  app_url_enc <- utils::URLencode(redirect_uri, reserved = TRUE)
  logout_url <- sprintf("%s/v2/logout?client_id=%s&returnTo=%s",
                        config$auth0_config$api_url,
                        config$auth0_config$credentials$key,
                        app_url_enc)
  logout_url
}

#' Log out of an auth0 app
#'
#' Log the current user out of an auth0 shiny app.
#'
#' @seealso \code{\link[auth0]{logoutButton}},
#' \code{\link[auth0]{logout_url}}
#'
#' @details You can also use a diferent configuration file by setting the
#' `auth0_config_file` option with:
#' `options(auth0_config_file = "path/to/file.yaml")`.
#'
#' @export
logout <- function() {

  if (!requireNamespace("shinyjs", quietly = TRUE)) {
    stop("Package \"shinyjs\" required.", call. = FALSE)
  }

  shiny::insertUI(
    selector = "head", where = "beforeEnd", immediate = TRUE,
    ui = shinyjs::useShinyjs()
  )
  url <- logout_url()
  js <- sprintf('location.replace("%s")', url)
  shinyjs::runjs(js)
}

---
title: "auth0 Tutorial"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{simple}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The goal of `{auth0}` is to implement an authentication scheme to Shiny using OAuth Apps through the freemium service [Auth0](https://auth0.com).

To create your authenticated shiny app, you need to follow the steps below. The full decription can be found in the `README` of the [package site](https://curso-r.github.io/auth0/).

### Create an Auth0 account and application

- Go to [auth0.com](https://auth0.com)
- Click "Sign Up"
- You can create an account with a user name and password combination, or by signing up with your GitHub or Google accounts.
- Click on "+ Create Application"
- Give a name to your app
- Select "Regular Web Applications" and click "Create"

### Configure your application

- Go to the Settings in your selected application. 
- Add `http://localhost:8080` to the "Allowed Callback URLs", "Allowed Web Origins" and "Allowed Logout URLs".
    - You can change `http://localhost:8080` to another port.
- Add the remote server where you are going to deploy your shiny app to the same boxes.
    - Just make sure that these addresses are correct. If you are placing your app inside a folder (e.g. `your_username.shinyapps.io/fooBar`), don't include the folder (`fooBar`) in "Allowed Web Origins".
- Click "Save"

Now, let's go to R!

### Fill the `_auth0.yml` file

- Create a configuration file for your shiny app by calling `auth0::use_auth0()`:

```r
auth0::use_auth0()
```

You can set the directory where this file will be created using the `path=` parameter. See `?auth0::use_auth0` for details. Your `_auth0.yml` file should be like this:


```yml
name: myApp
auth0_config:
  api_url: !expr paste0('https://', Sys.getenv("AUTH0_USER"), '.auth0.com')
  credentials:
    key: !expr Sys.getenv("AUTH0_KEY")
    secret: !expr Sys.getenv("AUTH0_SECRET")
```

Run `usethis::edit_r_environ()` and add these three environment variables:

```
AUTH0_USER=johndoe
AUTH0_KEY=5wugt0W...
AUTH0_SECRET=rcaJ0p8...
```

There's how you identify each of them (see the image below):`AUTH0_USER` is your username, which can be found on the top corner of the site. `AUTH0_KEY` is your Client ID, which can be copied from inside the app page. `AUTH0_SECRET` is your Client Secret, which can be copied from the app page.

You can find more information about environment variables [here](https://csgillespie.github.io/efficientR/set-up.html#renviron). You can also fill these information directly in the `_auth0.yml` file (see below). If you do so, don't forget to save the `_auth0.yml` file after editing it.

- Save and **restart your session**.

### Create your app

Write a simple shiny app in a `app.R` file, like this:

```r
library(shiny)

ui <- fluidPage(
  fluidRow(plotOutput("plot"))
)
  
server <- function(input, output, session) {
  output$plot <- renderPlot({
    plot(1:10)
  })
}

# note that here we're using a different version of shinyApp!
auth0::shinyAppAuth0(ui, server)
```

**Note**: If you want to use a different path to the `auth0` configuration file, you can either pass it to `shinyAppAuth0()` or set the `auth0_config_file` option by running `options(auth0_config_file = "path/to/file")`.

### Run!

You can try your app running

```r
options(shiny.port = 8080)
shiny::runApp("app/directory/")
```

If everything is OK, you should be forwarded to a login page and, after logging in or signing up, you'll be redirected to your app.

---
title: "ui-server-bookmark"
type: docs
repo: https://github.com/curso-r/auth0
menu:
  main:
    parent: examples
---

<div class="source-ref">
<span class="caption">Source: </span>`r sprintf("https://github.com/curso-r/auth0/blob/master/inst/%s", rmarkdown::metadata$title)`
</div>

Don't forget to configure your environment variables. 

Also, set the port, running something like `options(shiny.port = 8080)`.

## ui.R

```{r, echo = FALSE}
knitr::opts_chunk$set(eval = FALSE)

f <- system.file(sprintf("%s/ui.R", rmarkdown::metadata$title), package = "auth0")
knitr::spin_child(f)
```

## server.R

```{r, echo = FALSE, eval=TRUE}
knitr::opts_chunk$set(eval = FALSE)

f <- system.file(sprintf("%s/server.R", rmarkdown::metadata$title), package = "auth0")
knitr::spin_child(f)
```

## _auth0.yml

```{r, echo = FALSE, eval=TRUE, comment=NA}
f <- system.file(sprintf("%s/_auth0.yml", rmarkdown::metadata$title), package = "auth0")
cat(readLines(f), sep = "\n")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{shinyAuth0App}
\alias{shinyAuth0App}
\title{Create a Shiny app object with Auth0 Authentication}
\usage{
shinyAuth0App(ui, server, config_file = NULL)
}
\arguments{
\item{ui}{an ordinary UI object to create shiny apps.}

\item{server}{an ordinary server object to create shiny apps.}

\item{config_file}{path to YAML configuration file.}
}
\description{
As of auth0 0.1.2, \code{shinAuth0App()} has
been renamed to \code{\link[=shinyAppAuth0]{shinyAppAuth0()}} for consistency.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{auth0_logout_url}
\alias{auth0_logout_url}
\title{Generate logout URL}
\usage{
auth0_logout_url(config_file = NULL, redirect_js = TRUE)
}
\arguments{
\item{config_file}{Path to YAML configuration file.}

\item{redirect_js}{include javascript code to redirect page? Defaults to \code{TRUE}.}
}
\description{
\code{auth0_logout_url()} is defunct as of auth0 0.1.2 in order to simplifly the
user experience with the \code{\link[=logoutButton]{logoutButton()}} function.
}
\examples{
\donttest{

# simple UI with action button
# AFTER auth0 0.1.2

if (interactive()) {
  library(shiny)
  library(auth0)

  ui <- fluidPage(logoutButton())
  server <- function(input, output, session) {}
  config_file <- system.file("simple/_auth0.yml", package = "auth0")
  shinyAppAuth0(ui, server, config_file)

  # simple UI with action button
  # BEFORE auth0 0.1.2

  library(shiny)
  library(auth0)
  library(shinyjs)

  # note that you must include shinyjs::useShinyjs() for this to work
  ui <- fluidPage(shinyjs::useShinyjs(), actionButton("logout_auth0", "Logout"))

  # server with one observer that logouts
  server <- function(input, output, session) {
    observeEvent(input$logout_auth0, {
      # javascript code redirecting to correct url
      js <- auth0_logout_url()
      shinyjs::runjs(js)
    })
  }
  shinyAuth0App(ui, server, config_file)
}

}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{ui-server}
\alias{ui-server}
\alias{auth0_ui}
\alias{auth0_server}
\title{Modifies ui/server objects to authenticate using Auth0.}
\usage{
auth0_ui(ui, info)

auth0_server(server, info, remove_callback_params = TRUE)
}
\arguments{
\item{ui}{\code{shiny.tag.list} object to generate the user interface.}

\item{info}{object returned from \link{auth0_info}. If not informed,
will try to find the \verb{_auth0.yml} and create it automatically.}

\item{server}{the shiny server function.}

\item{remove_callback_params}{whether to remove the \code{code} and \code{state} query.}
}
\description{
These functions can be used in a ui.R/server.R framework, modifying the
shiny objects to authenticate using Auth0 service with no pain.
}
\examples{
\donttest{
# first, create the yml file using use_auth0() function

if (interactive()) {

  # ui.R file
  library(shiny)
  library(auth0)
  auth0_ui(fluidPage(logoutButton()))

  # server.R file
  library(auth0)
  auth0_server(function(input, output, session) {})

  # console
  options(shiny.port = 8080)
  shiny::runApp()
}

}
}
\seealso{
\link{auth0_info}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logout.R
\name{logout}
\alias{logout}
\title{Log out of an auth0 app}
\usage{
logout()
}
\description{
Log the current user out of an auth0 shiny app.
}
\details{
You can also use a diferent configuration file by setting the
\code{auth0_config_file} option with:
\code{options(auth0_config_file = "path/to/file.yaml")}.
}
\seealso{
\code{\link[auth0]{logoutButton}},
\code{\link[auth0]{logout_url}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth0.R
\name{use_auth0}
\alias{use_auth0}
\title{Auth0 configuration file}
\usage{
use_auth0(path = ".", file = "_auth0.yml", overwrite = FALSE)
}
\arguments{
\item{path}{Directory name. Should be the root of the shiny app
you want to add this functionality}

\item{file}{File name. Defaults to \verb{_auth0.yml}.}

\item{overwrite}{Will only overwrite existing path if \code{TRUE}.}
}
\description{
Create an YAML containing information to connect with Auth0.
}
\details{
The YAML configuration file has required parameters and extra
parameters.

The required parameters are:
\itemize{
\item \code{auth0_config} is a list contaning at least:
\itemize{
\item \code{api_url}: Your account at Auth0 (e.g. https://jonhdoe.auth0.com).
It is the "Domain" in Auth0 application settings.
\item \code{credentials}: Your credentials to access Auth0 API, including
\itemize{
\item \code{key}: the Client ID in Auth0 application settings.
\item \code{secret}: the Client Secret in Auth0 application settings.
}
}
}

The extra parameters are:
\itemize{
\item \code{remote_url}: If you are using Shiny-Server or ShinyApps IO service.
\item \code{scope}: The information that Auth0 app will access.
Defaults to "openid profile".
\item \code{request}: Endpoit to request a token. Defaults to "oauth/token"
\item \code{access}: Endpoit to access. Defaults to "oauth/token"
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{shinyAppAuth0}
\alias{shinyAppAuth0}
\title{Create a Shiny app object with Auth0 Authentication}
\usage{
shinyAppAuth0(
  ui,
  server,
  config_file = NULL,
  remove_callback_params = TRUE,
  ...
)
}
\arguments{
\item{ui}{an ordinary UI object to create shiny apps.}

\item{server}{an ordinary server object to create shiny apps.}

\item{config_file}{path to YAML configuration file.}

\item{remove_callback_params}{whether to remove the \code{code} and \code{state} query
parameters from the URL after successful authentication. Defaults to \code{TRUE}.}

\item{...}{Other arguments passed on to \code{\link[shiny:shinyApp]{shiny::shinyApp()}}.}
}
\description{
This function modifies ui and server objects to run using Auth0
authentication.
}
\details{
You can also use a diferent configuration file by setting the
\code{auth0_config_file} option with:
\code{options(auth0_config_file = "path/to/file.yaml")}.
}
\section{Disable auth0 while developing apps}{


Sometimes, using auth0 to develop and test apps can be frustrating,
because every time the app is started, auth0 requires the user to log-in.
To avoid this problem, one can run \code{options(auth0_disable = TRUE)} to
disable auth0 temporarily.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{auth0_find_config_file}
\alias{auth0_find_config_file}
\title{Find the configuration file.}
\usage{
auth0_find_config_file()
}
\value{
Character vector of length one contaning the path of the
\verb{_auth0.yml} file.
}
\description{
Tries to find the path to the \verb{_auth0.yml} file. First, it tries to get
this info from \code{options(auth0_config_file = )}. If this option is \code{NULL}
(the default) it tries to find the \verb{_auth0.yml} within the working
directory. If the file does not exist, it raises an error.
}
\seealso{
\code{\link{use_auth0}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logout.R
\name{logoutButton}
\alias{logoutButton}
\title{Create a button to log out}
\usage{
logoutButton(label = "Log out", ..., id = "._auth0logout_")
}
\arguments{
\item{label}{The label on the button.}

\item{...}{Named attributes to apply to the button.}

\item{id}{An ID for the button. If you only have one logout button in
your app, you do not need to explicitly provide an ID. If you have more than
one logout button, you need to provide a unique ID to each button. When you
create a button with a non-default ID, you must create an observer that
listens to a click on this button and logs out of the app with a call to
\code{\link[auth0]{logout}}.}
}
\description{
A \code{logoutButton} is an \code{\link[shiny]{actionButton}} that is meant
to be used to log out of an auth0 Shiny app.
}
\examples{
if (interactive()) {
  ui <- fluidPage(
    logoutButton(),
    logoutButton(label = "Another logout button", id = "logout2")
  )
  server <- function(input, output, session) {
    observeEvent(input$logout2, {
      logout()
    })
  }
  shinyAuth0App(ui, server)
}

}
\seealso{
\code{\link[auth0]{logout}},
\code{\link[auth0]{logout_url}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth0.R
\name{auth0_info}
\alias{auth0_info}
\title{Information used to connect to Auth0.}
\usage{
auth0_info(config)
}
\arguments{
\item{config}{path to the \verb{_auth0.yml} file or the object returned by
\link{auth0_config}. If not informed, will try to find the file using
\link{auth0_find_config_file}.}
}
\value{
A list contaning scope, state, keys, OAuth2.0 app, endpoints,
audience and
remote URL. For compatibility reasons, \code{remote_url} can be either a parameter
in the root yaml or inside a \code{shiny_config} parameter.
}
\description{
Creates a list containing all the important information to connect to Auth0
service's API.
}
\seealso{
\link{use_auth0} to create an \verb{_auth0.yml} template.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logout.R
\name{logout_url}
\alias{logout_url}
\title{Generate a logout URL}
\usage{
logout_url()
}
\value{
URL string to log out.
}
\description{
Generate a URL that will log the user out of the app if visited.
}
\details{
You can also use a diferent configuration file by setting the
\code{auth0_config_file} option with:
\code{options(auth0_config_file = "path/to/file.yaml")}.
}
\seealso{
\code{\link[auth0]{logoutButton}},
\code{\link[auth0]{logout}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auth0.R
\name{auth0_config}
\alias{auth0_config}
\title{Parse \verb{_auth0.yml} file.}
\usage{
auth0_config(config_file)
}
\arguments{
\item{config_file}{path to the \verb{_auth0.yml} file. If not informed,
will try to find the file using \link{auth0_find_config_file}.}
}
\value{
List containing all the information from the \verb{_auth0.yml} file.
}
\description{
Validates and creates a list of useful information from
the \verb{_auth0.yml} file.
}
